/*
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */

package com.openexchange.mobile.api.facade.connectors.impl;

import java.nio.charset.Charset;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.openexchange.mobile.api.facade.auth.SessionData;
import com.openexchange.mobile.api.facade.connectors.BaseConnector;
import com.openexchange.mobile.api.facade.connectors.RequestConfiguration;
import com.openexchange.mobile.api.facade.connectors.requests.mtos.MailAccountDataMto;
import com.openexchange.mobile.api.facade.connectors.responses.MailAccountResponseMto;
import com.openexchange.mobile.api.facade.models.MailAccountData;
import com.openexchange.mobile.api.facade.utils.JacksonUtil;

import lombok.SneakyThrows;

import org.apache.http.HttpEntity;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.entity.StringEntity;

public abstract class MailAccountBaseConnector extends BaseConnector<MailAccountResponseMto> {

    private final String action;

    private final MailAccountDataMto accountData;

    public MailAccountBaseConnector(RequestConfiguration configuration, SessionData sessionData, String action, MailAccountData account) {
        super(configuration, sessionData, "account", MailAccountResponseMto.class);
        this.action = action;
        this.accountData = new MailAccountDataMto(account);
    }

    @Override
    protected HttpUriRequest createRequest() {
        RequestBuilder builder = RequestBuilder.put(createUrl());
        builder.setCharset(Charset.forName("UTF-8"));
        addSessionData(builder);
        builder.addParameter("action", action);
        builder.setEntity(createEntity());
        return builder.build();
    }

    @SneakyThrows
    private HttpEntity createEntity() {
        ObjectMapper mapper = JacksonUtil.getMapper();
        String str = mapper.writeValueAsString(accountData);
        return new StringEntity(str, "UTF-8");
    }

    @Override
    protected boolean isError(MailAccountResponseMto responseBody) {
        if (responseBody != null && "WARNING".equals(responseBody.getCategories())) {
            return false;
        }
        return super.isError(responseBody);
    }

    public static MailAccountData map(MailAccountResponseMto response) {
        if (response == null || response.getData() == null) {
            return null;
        }
        return new MailAccountData(response.getData());
    }

}